'''
Query the Deployed Connect API for Zoltar Credential.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports
import os
from plinko.configurations import configure_username
from plinko.configurations import configure_environment

def zoltar_ask(wish):
    '''
    Query Zoltar API for secure credential by passing "Wish" to connect_server
    
    Parameters
    ----------
    wish : str
        String identifier to query Zoltar API with
        
    Returns
    -------
    str
        Result from Zoltar API query
    '''
    # Check for environment configuration
    if (os.getenv("CONNECT_API_KEY") is None) or (os.getenv("CONNECT_USERNAME") is None): configure_username()

    # Check for connect server configuration
    if (os.getenv("CONNECT_SERVER") is None) or (os.getenv("CONNECT_SERVER_TYPE") is None): configure_environment()
    
    # Query API
    curl = f'''curl -s -X GET "{os.getenv("CONNECT_SERVER")}/zoltar/wish?wish={wish}" -H "Authorization: Key {os.getenv('CONNECT_API_KEY')}" -H "accept: */*"'''
    x    = os.popen(curl).read()
    
    # Return query result
    return eval(x)[0]



'''
Set CONNECT_USER Environment Variable.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports
import os
from .configure_api_key import configure_api_key
from .configure_environment import configure_environment


def configure_username():
    '''
    Configure CONNECT_USER environment variable in $PATH for assessing
    server-level permissions when authenticating to Connect servers
    
    Parameters
    ----------
    None
        
    Returns
    -------
    int
       1 if CONNECT_USER variable can be set to $PATH by querying connect_server's user API,
       else 0
    '''
    # Configure API key if not configured
    if os.getenv("CONNECT_API_KEY") is None: configure_api_key()

    # Configure server information if it is not configured
    if (os.getenv("CONNECT_SERVER") is None) or (os.getenv("CONNECT_SERVER_TYPE") is None): configure_environment()
    
    # Call user API for user information
    curl = f'''curl -s -X GET {os.getenv("CONNECT_SERVER")}/__api__/v1/user -H "Authorization: Key {os.getenv('CONNECT_API_KEY')}" -H "accept: */*"'''
    x    = os.popen(curl).read()
    if x == '' or x is None:
        print("Unable to establish user identification. Please verify CONNECT_API_KEY and try again.")
        return 1
    
    # Post process API return
    true  = True
    false = False
    x     = eval(x)
    
    # Check for valid response with username information
    if "username" in x.keys():
        os.environ["CONNECT_USER"] = x["username"]
        
        # By default, lock user access
        if "locked" in x.keys(): os.environ["CONNECT_USER_LOCKED"] = str(x["locked"])
        else: os.environ["CONNECT_USER_LOCKED"] = "True"
        return 0
    
    print("Unable to establish user identification. Please verify CONNECT_API_KEY and try again.")
    return 1


'''
Connect to Amazon AWS S3 Storage (Cold Storage).

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports
import os
import boto3
from plinko.zoltar import zoltar_ask
from plinko.configurations import configure_username


def plinko_s3R():
    '''
    Establish needed environment variables and global environment variables
    for connecting and reading/writing data to AWS S3 storage (Cold Storage)

    Parameters
    ----------
    None

    Returns
    -------
    dict
        Dicitonary containing s3BucketName and connection object from boto3.client()
    '''
    # Check for API Key configuration
    if os.getenv("CONNECT_API_KEY") is None: configure_username()

    # Check for existing AWS S3 configuration information
    if os.getenv("AWS_SECRET_ACCESS_KEY") is not None: 
        print("You already have a defined S3 connection configured in your global environment.")
        print("You can only assume one role at a time. You must restart your session to connect to a different S3 storage location.")
        return None

    # Define configuration
    os.environ["AWS_ACCESS_KEY_ID"]     = zoltar_ask("aws_key")
    os.environ["AWS_SECRET_ACCESS_KEY"] = zoltar_ask("aws_secretkey") 
    os.environ["AWS_DEFAULT_REGION"]    = "us-east-1"

    # Return bucket name and session client for accessing data
    return {"s3BucketName": zoltar_ask("s3BucketName"), "conn": boto3.client("s3")} 


'''
Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''

from .plinko_hadoop import plinko_hadoop
from .plinko_sql import plinko_sql
from .plinko_s3R import plinko_s3R
from .plinko_s3 import plinko_s3
from .plinko_redshift import plinko_redshift
from .plinko_postgres import plinko_postgres


'''
Configuration to determine environment user is operating in.

Copyright (C) 2024 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''

# Imports 
import os

def configure_environment():
    '''
    Add CONNECT_SERVER to environment variables in order to authenticate 
    to Connect servers and call proper zoltar API endpoint. Also sets
    CONNECT_SERVER_TYPE to either PROD or DEV to control downstream flows.
    
    Parameters
    ----------
    None
    
    Returns
    -------
    int
        1 if CONNECT_SERVER and CONNECT_SERVER_TYPE environment variables can be set, 
            else 0
    '''
    # Check to ensure environment is not set
    if (not os.getenv("CONNECT_SERVER") is None) and (not os.getenv("CONNECT_SERVER_TYPE") is None):
        return 0

    # Check for hostname
    hostname_ = os.popen('hostname').read().strip()

    # Verify server and set environment accordingly
    connect_server_      = None
    connect_server_type_ = None

    # Kortex environment - DEV
    if hostname_ in ['usaws3320', 'usaws3321', 'usaws3322', 'usaws3323']:
        connect_server_      = "https://dev.positconnect.analytics.kellogg.com"
        connect_server_type_ = "DEV"

    # Kortex environment - PROD
    if hostname_ in ["usaws1320", "usaws1321", "usaws1322", "usaws1323"]:
        connect_server_      = "https://prod.positconnect.analytics.kellogg.com"
        connect_server_type_ = "PROD"

    # Legacy environment - PROD
    if hostname_ in ["usaws1170", "usaws1171", "usaws1172", "usaws1173", "usaws1174", "usaws1175"]:
        connect_server_      = "https://rstudioconnect.analytics.kellogg.com"
        connect_server_type_ = "PROD"

    # NOTE: Legacy environment - DEV is not supported
    if hostname_ in ["usaws3170", "usaws3171", "usaws3172", "usaws3173"]:
        raise NotImplementedError("Zoltar is currently not supported in this environment")

    # Default to Kortex environment - PROD for all other executions
    if (connect_server_ is None) and (connect_server_type_ is None):
        connect_server_      = "https://prod.positconnect.analytics.kellogg.com"
        connect_server_type_ = "PROD"

    # Set environment variables
    os.environ['CONNECT_SERVER']      = connect_server_
    os.environ['CONNECT_SERVER_TYPE'] = connect_server_type_

    # Check environment variables
    if (os.getenv("CONNECT_SERVER") == connect_server_) and (os.getenv("CONNECT_SERVER_TYPE") == connect_server_type_):
        return 1
    return 0


'''
Connect to Amazon Redshift Databases.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports 
import os
import pyodbc
from plinko.configurations import configure_username
from plinko.zoltar import zoltar_ask


def plinko_redshift(environment, region = 'KNA',
                    database = None, server = None):
    '''
    Connect to AWS Redshift databases in Kortex

    Parameters
    ----------
    environment: str
        Kortex environment to connect to (DEV, PROD, QA)
    region: str
        Regional redshift instance to connect to. Can be 'kna', 'keu', 'kamea'
        Default 'kna'
    database: str (optional)
        Specify database to connect to in Kortex environment. If None, REDSHIFT_default_database
        information is pulled from zoltar
    server: str (optional)
        Specify custom server to connect to in Kortex environment. If None, server is determined
        based on environment

    Returns
    -------
    pyodbc.connect
        If connection can be established using pyodbc.connect(), a pyodbc.connect() object
        will be returned. This can be passed to pandas (e.g., pd.read_sql) or similar libraries
        for reading remote data. If a connection cannot be established, None will be returned
    '''
    # Check for environment variable configurations
    if os.getenv("CONNECT_API_KEY") is None: configure_username()

    # Correct user input for AMEA
    if region.lower() in ['kamea', 'kap']: region = 'AMEA'

    # Query relevant information from zoltar
    redshift_id = f'REDSHIFT_{region.upper()}_{environment.upper()}'
    server_id   = f'{redshift_id}_server'
    port_id     = f'{redshift_id}_port'
    uid_id      = f'{redshift_id}_uid'
    pwd_id      = f'{redshift_id}_pwd'

    # Define database if none passed
    if database is None:
        if region == 'AMEA':
            database = 'klg_nga_kamea'
        else:
            database = f'klg_nga_{region.lower()}'

    # Establish connection
    cnxn = pyodbc.connect(Driver = 'redshift',
                          servername = zoltar_ask(server_id),
                          database = database,
                          UID = zoltar_ask(uid_id),
                          PWD = zoltar_ask(pwd_id),
                          Port = zoltar_ask(port_id))

    return cnxn


from setuptools import setup, find_packages

VERSION = "0.2.1"

setup(
    name = 'plinko',
    version = VERSION,
    author = 'Nat Hawkins, Mike Chappelow',
    author_email = 'nat.hawkins@kellogg.com, michael.chappelow@kellogg.com',
    description = 'Python equivalent to the internal R library, klink, for establishing remote connections to data sources',
    license = 'LICENSE.txt',
    packages = find_packages(),
    scripts = [],
    install_requires=[],
    url = 'https://pypi.org/project/plinko'
)


'''
Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''

from .zoltar_ask import zoltar_ask
from .zoltar_list import zoltar_list


'''
Connect to Hadoop Server.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports
import os
import pyodbc
from plinko.zoltar import zoltar_ask
from plinko.configurations import configure_username


def plinko_hadoop(environment, schema):
    '''
    Connect to Hadoop server using ODBC Hive driver
    
    Parameters
    ----------
    environment: str
        String denoting Hadoop server to connect to, can be either DEV or PROD
    schema: str
        Database/schema in Hadoop server to access
        
    Returns
    -------
    pyodbc.connect  
        If connection can be established using pyodbc.connect(), a pyodbc.connect() object
        will be returned. This can be passed to pandas (e.g., pd.read_sql) or similar libraries
        for reading remote data. If a connection cannot be established, None will be returned
    '''
    # Check for environment variable
    if os.getenv("CONNECT_API_KEY") is None: configure_username()
    
    # Check for Hive drivers
    if 'Hive' not in pyodbc.drivers():
        print("Your system currently lacks the necessary Hive driver for connecting to Hadoop.")
        return None
        
    # Get service account credentials
    username = zoltar_ask(f'{environment}_HADOOP_userid')
    password = zoltar_ask(f'{environment}_HADOOP_pwd')

    # Run kerboros authentication
    kerboros_status = os.system(f"kinit {username} <<< {password}")
    if kerboros_status != 0:
        print("Kerboros authentication failed.")
        return None

    # Get Hadoop server information
    host = zoltar_ask(f'{environment}_HADOOP_server')
    port = zoltar_ask(f'{environment}_HADOOP_port')
    
    # Identify SSl certificate location for environment
    if os.getenv("CONNECT_SERVER_TYPE") == "PROD":
        if "prod.positconnect." in os.getenv("CONNECT_SERVER"):
            ssl_cert = "/usr/rstudio_prod/serverpro/certs/hive_prod.pem"
        else:
            ssl_cert = "/usr/rstudio/serverpro/certs/hive_prod.pem"
    else:
        ssl_cert = "/usr/rstudio2/serverpro/certs/hive.pem"
    
    # Connect
    cnxn = pyodbc.connect("DRIVER={Hive};" +
                          f"HOST={host};PORT={port};DATABASE={schema};"+
                          "ZKNamespace=None;"+
                          "AuthMech=1;"+
                          "ServiceDiscoveryMode=0;"+
                          "HiveServerType=2;"+
                          "ThriftTransport=1;"+
                          "SSL=1;"+
                          f"ClientCert={ssl_cert};"+
                          "AllowSelfSignedServerCert=1;"+
                          "HttpPathPrefix=/cliservice",
                          autocommit = True)
    return cnxn


'''
Set CONNECT_API_KEY Environment Variable from .Renviron File.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports 
import os


def configure_api_key():
    '''
    Add CONNECT_API_KEY to $PATH in order to authenticate to Connect servers
    
    Parameters
    ----------
    None
    
    Returns
    -------
    int
        1 if CONNECT_API_KEY environment variable is located in a .Renviron 
        file either in current working directory or home directory, else 0
    '''
    # If environment variable is not set, os.getenv("<variable name>")
    # will return None. If it is not None, the API key is recognized
    # and set in the evironment.
    if not os.getenv("CONNECT_API_KEY") is None: return 0
    
    # If environment is not set, checks the current working directory and
    # home directory for a .Renviron file and read CONNECT_API_KEY from there
    for target_filename in [os.path.expanduser(os.getcwd() + "/.Renviron"), 
                            os.path.expanduser("~" + "/.Renviron")]:
        # Check file exists
        if not os.path.exists(target_filename): continue        
    
        # Read .Renviron file 
        with open(target_filename, "r") as f:
            # Find variable labeled CONNECT_API_KEY
            environment_variables = f.readlines()
            connect_api_key       = [_x for _x in environment_variables if "CONNECT_API_KEY" in _x]

        if len(connect_api_key) == 0: continue
        
        connect_api_key = connect_api_key[0].split("=")[-1].strip()
        if "'" in connect_api_key or '"' in connect_api_key: 
            connect_api_key = connect_api_key[1:-1]
        
        # Set environment variable 
        os.environ["CONNECT_API_KEY"] = connect_api_key
        return 0
                
    # At this point in the execution, no CONNECT_API_KEY has been found
    print("After checking your home directory and your current working directory, " \
          "there does not appear to be a .Renviron file with CONNECT_API_KEY present. " \
          "Please ensure file exists and contains an API key and try again.")
    return 1
    


'''
Connect to Amazon AWS S3 Storage (Kortex) by Assuming IAM Role.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports
import os
import boto3
from plinko.zoltar import zoltar_ask
from plinko.configurations import configure_username


def plinko_s3(ignore_existing = False, region = 'KNA'):
    '''
    Establish needed environment variables and global environment variables
    for connecting and reading/writing data to AWS S3 storage (Kortex) by assuming
    IAM Role

    Parameters
    ----------
    ignore_existing: bool
        Boolean value indicating whether to ignore existing connections. Default False
    region: str
        String indicating regional bucket. Can be 'global', 'kna', 'keu', 'kamea', 'kla', 'kap'. Default 'kna'

    Returns
    -------
    dict
        Dicitonary containing s3BucketName and connection object from boto3.client()
    '''
    # Check for API Key configuration
    if os.getenv("CONNECT_API_KEY") is None: configure_username()

    # Check for existing AWS S3 configuration information
    if (ignore_existing == False) and (os.getenv("AWS_SECRET_ACCESS_KEY") is not None): 
        print("You already have a defined S3 connection configured in your global environment.")
        print("You can only assume one role at a time. You must restart your session to connect to a different S3 storage location.")
        return None

    # Create placeholder for Session to assume IAM role for
    # Same for PROD and DEV
    session = boto3.Session()
    sts     = session.client('sts')

    # DEV - Kortex only supported DEV instance
    if os.getenv("CONNECT_SERVER_TYPE") == 'DEV':
        bucket_name = zoltar_ask(f"s3BucketName_{region.lower()}_kortex_dev")
        response    = sts.assume_role(RoleArn = zoltar_ask(f"s3iam_{region.lower()}_kortex_dev"),
                                      RoleSessionName = "rstudio")

    # PROD - Kortex and Legacy environment supported
    else:
        # Legacy system
        if 'rstudioconnect' in os.getenv('CONNECT_SERVER'):
            bucket_name = zoltar_ask("s3BucketName_kortex_DEV")
            response    = sts.assume_role(RoleArn = zoltar_ask("s3iam_posit_keystone_DEV"),
                                          RoleSessionName = "rstudio")
        else:
            bucket_name = zoltar_ask(f"s3BucketName_{region.lower()}_kortex_prod")
            response    = sts.assume_role(RoleArn = zoltar_ask(f"s3iam_{region.lower()}_kortex_prod"),
                                          RoleSessionName = "rstudio")

    # Create session with temporary credentials from assuming IAM role
    temporary_credentials = boto3.Session(aws_access_key_id = response['Credentials']['AccessKeyId'],
                                          aws_secret_access_key = response['Credentials']['SecretAccessKey'],
                                          aws_session_token = response['Credentials']['SessionToken'])

    # Return bucket name and session client for accessing data
    return {"s3BucketName": bucket_name, "conn": temporary_credentials.client("s3")}



'''
Connect to Postgres Databases.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports 
import os
import pyodbc
from plinko.configurations import configure_username
from plinko.zoltar import zoltar_ask


def plinko_postgres(environment, database, server = None):
    '''
    Connect to Postgres SQL Database

    Parameters
    ----------
    environment: str
        Postgres SQL environment to connect to
    database: str
        Specify database in environment to connect to
    server: str (optional)
        Specify server to use when connection to Postgres SQL. If None, server information is
        determined based on environment and database parameters

    Returns
    -------
    pyodbc.connect
        If connection can be established using pyodbc.connect(), a pyodbc.connect() object
        will be returned. This can be passed to pandas (e.g., pd.read_sql) or similar libraries
        for reading remote data. If a connection cannot be established, None will be returned
    '''
    # Check for environment variable configurations
    if os.getenv("CONNECT_API_KEY") is None: configure_username()

    # DEV
    if environment.upper() == "DEV":
        if server is None:
            server = zoltar_ask("POSTGRES_DEV_server")
        
        port = zoltar_ask("POSTGRES_DEV_port")
        uid  = zoltar_ask("POSTGRES_DEV_uid")
        pwd  = zoltar_ask("POSTGRES_DEV_pwd")

    # PROD
    elif environment.upper() == "PROD":
        print("PROD credentials are not currently supported")
        return None

    else:
        print("Please specify PROD or DEV as your environment.")
        return None

    # Establish connection
    cnxn = pyodbc.connect(Driver = 'postgresql',
                          Server = server,
                          Port = port,
                          Database = database,
                          UID = uid, 
                          PWD = pwd,
                          BoolAsChar = "",
                          timeout = 10)

    return cnxn


'''
Query the Deployed Connect API for all Available Zoltar Credentials.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''

 
# Imports
import os
import pandas as pd
from plinko.plinko_sql import plinko_sql
from plinko.configurations import configure_username
 
def zoltar_list():
    '''
    Get list of all available credentials in Zoltar API
    
    Parameters
    ----------
    None
    
    Returns
    -------
    pandas.data.frame
        pandas.data.frame object containing all available credentials in Zoltar API
    '''
    # Check for environment variables
    if os.getenv("CONNECT_USER_LOCKED") is None: configure_username()
    
    # Check if user is locked
    if os.getenv("CONNECT_USER_LOCKED") == "True" or os.getenv("CONNECT_USER_LOCKED") is None:
        return None
        
    # TODO: Add more robust user role checking and user-level permissions
    # for restricting super access to credentials
        
    # Connect to credentials table in SQL
    cnxn = plinko_sql("PROD", "KG_R_APPS")
    
    # Check to see if access can be established
    if cnxn is None:
        return None
        
    # Return credentials
    return pd.read_sql("SELECT * FROM dbo.zoltar_list", cnxn)


'''
Connect to SQL Databases.

Copyright (C) 2023 Nat Hawkins & Mike Chappelow

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License (version 3 of the License)
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''


# Imports
import os
import pyodbc
from plinko.configurations import configure_username
from plinko.zoltar import zoltar_ask


def plinko_sql(environment, database, server = None):
    '''
    Access and connect to SQL database in specified environment
    
    Parameters
    ----------
    environment: str
        SQL Environment to access, can be either DEV or PROD
    database: str
        Database within SQL to access
    server: str (optional)
        Custom server to use rather than default MS SQL Server locations
        
    Returns
    -------
    pyodbc.connect
        If connection can be established using pyodbc.connect(), a pyodbc.connect() object
        will be returned. This can be passed to pandas (e.g., pd.read_sql) or similar libraries
        for reading remote data. If a connection cannot be established, None will be returned
    '''
    # Check for configuration of environment variables
    if os.getenv("CONNECT_API_KEY") is None: configure_username()
    
    # Determine driver to use for connection
    available_drivers = pyodbc.drivers()
    
    # Default to freetds for full functionality, otherwise use SQLServer/SQL Server
    if 'freetds' in available_drivers or 'FreeTDS' in available_drivers:
        unsupported_service_accounts_present = False
        driver = 'freetds' if 'freetds' in available_drivers else 'FreeTDS'
    else:
        unsupported_service_accounts_present = True
        if 'SQL Server' in available_drivers:
            driver = 'SQL Server'
        else:
            driver = 'SQLServer'
    
    # Specified custom server
    if server is not None:
        server_val = server
        
        # Custom server DEV
        if environment == "DEV":
            if "USAWSCWSQL5066" in server.upper():
                uid = zoltar_ask("USAWSCWSQL5066_DEV_userid")
                pwd = zoltar_ask("USAWSCWSQL5066_DEV_pwd")
                
        # Custom server PROD
        elif environment == "PROD":
            if database.upper() == "KG_ANALYTICS_APPS":
                if "USAWSCWSQL0066" in server.upper():
                    uid = zoltar_ask("Usawscwsql0066_KG_ANALYTICS_APPS_PROD_uid")
                    pwd = zoltar_ask("Usawscwsql0066_KG_ANALYTICS_APPS_PROD_pwd")
                else:
                    uid = zoltar_ask("Usawscwsql0066_KG_SAS_PROD_userid")
                    pwd = zoltar_ask("Usawscwsql0066_KG_SAS_PROD_pwd")
        
        # Cannot connect
        else:
            print("Unknown request, please contact Data Science team for support.")
            return None
    
    # Default server DEV
    elif environment == "DEV":
        server_val = zoltar_ask("MS_SQL_ANALYTICS_DEV_server")
        
        if database in ["KG_R_APPS", "KNA_FIN"]:
            if unsupported_service_accounts_present:
                print("Due to your available ODBC drivers, this service account cannot be utilized.")
                return None
            uid = zoltar_ask("KG_R_APPS_DEV_userid")
            pwd = zoltar_ask("KG_R_APPS_DEV_pwd")
            
        elif database in ["KG_SAS", "KG_SC", "KG_EXTERNAL", "KNA_ECC", "KG_SANDBOX", "KG_MEMSQL", "KG_VIEWS"]:
            uid = zoltar_ask("KG_SAS_DEV_userid")
            pwd = zoltar_ask("KG_SAS_DEV_pwd")
        
        elif database in ["KG_SAS_WRITE"]:
            database = "KG_SAS" 
            uid = zoltar_ask("KG_SAS_DEV_WRITE_uid")
            pwd = zoltar_ask("KG_SAS_DEV_WRITE_pwd")
        
        elif database in ["KG_ANALYTICS_APPS", "WKKC_K_ANALYTICS_APPS"]:
            uid = zoltar_ask("KG_ANALYTICS_APPS_DEV_userid")
            pwd = zoltar_ask("KG_ANALYTICS_APPS_DEV_pwd")
            
        else:
            uid = zoltar_ask("KG_SAS_DEV_userid")
            pwd = zoltar_ask("KG_SAS_DEV_pwd")
        
    # Default server PROD
    elif environment == "PROD":
        server_val = zoltar_ask("MS_SQL_ANALYTICS_PROD_server")
        
        if database in ["KG_R_APPS", "KNA_FIN"]:
            if unsupported_service_accounts_present:
                print("Due to your available ODBC drivers, this service account cannot be utilized.")
                return None
            uid = zoltar_ask("KG_R_APPS_PROD_userid")
            pwd = zoltar_ask("KG_R_APPS_PROD_pwd")
            
        elif database in ["KG_SAS", "KG_SC", "KG_EXTERNAL", "KNA_ECC", "KG_SANDBOX", "KG_MEMSQL", "KG_VIEWS"]:
            uid = zoltar_ask("KG_SAS_PROD_userid")
            pwd = zoltar_ask("KG_SAS_PROD_pwd")
            
        elif database in ["KG_SAS_WRITE"]:
            database = "KG_SAS" 
            uid = zoltar_ask("KG_SAS_PROD_WRITE_uid")
            pwd = zoltar_ask("KG_SAS_PROD_WRITE_pwd")
            
        elif database in ["KG_ANALYTICS_APPS", "WKKC_KG_ANALYTICS_APPS"]:
            uid = zoltar_ask("KG_ANALYTICS_APPS_PROD_userid")
            pwd = zoltar_ask("KG_ANALYTICS_APPS_PROD_pwd")
            
        else:
            uid = zoltar_ask("KG_SAS_PROD_userid")
            pwd = zoltar_ask("KG_SAS_PROD_pwd")
        
    # Cannot connect
    else:
        print("Could not establish connect with given parameters, please try again.")
        return None
        
    # Create connection
    cnxn = pyodbc.connect(f'Driver={{{driver}}};',
                          Server = server_val,
                          Database = database,
                          User = uid,
                          Password = pwd,
                          Encrypt = 'yes',
                          Trusted_Connection = 'no')
    return cnxn
